#include <iostream>
#include <iomanip>
#include <vector>
#include <queue>
#include <algorithm>
#include <functional>

using namespace std;

// Increase a digit.
int increaseDigit(int x) {
	return (x + 1) % 10;
}

// Decrease a digit (note: modulo in C++ behaves strange with negative numbers, so we add 9 instead
// of subtracting 1, which is equivalent)
int decreaseDigit(int x) {
	return (x + 9) % 10;
}

// Increase the two least significant digits.
int increaseLow(int original) {
	int high = original / 100;
	int mid = (original / 10) % 10;
	int low = original % 10;

	return high*100 + increaseDigit(mid)*10 + increaseDigit(low);
}

// Increase the two most significant digits.
int increaseHigh(int original) {
	int high = original / 1000;
	int mid = (original / 100) % 10;
	int low = original % 100;

	return increaseDigit(high)*1000 + increaseDigit(mid)*100 + low;
}

// Decrease the two middle digits.
int decreaseMid(int original) {
	int a = original / 1000;
	int b = (original / 100) % 10;
	int c = (original / 10) % 10;
	int d = original % 10;

	return a*1000 + decreaseDigit(b)*100 + decreaseDigit(c)*10 + d;
}

vector<int> bfs(int from, int to) {
	vector<function<int(int)>> options{increaseHigh, increaseLow, decreaseMid};

	// Remember where we came from!
	vector<int> prev(10000, -1);

	queue<int> q;
	q.push(from);

	// Mark the starting node so we can recognize it later.
	prev[from] = from;

	while (!q.empty()) {
		int now = q.front(); q.pop();
		if (now == to)
			break;

		for (auto &&f : options) {
			int next = f(now);
			if (prev[next] < 0) {
				// Mark where we came from, bonus mark it as visited!
				prev[next] = now;
				q.push(next);
			}
		}
	}

	// Did we find any path?
	if (prev[to] < 0)
		return vector<int>();

	// Find the path by looking at the 'prev' array.
	vector<int> result;
	for (int at = to; prev[at] != at; at = prev[at]) {
		result.push_back(at);
	}
	result.push_back(from);

	reverse(result.begin(), result.end());

	return result;
}

int main(int argc, const char **argv) {
	int from = 0;
	int to = 3146;

	if (argc == 3) {
		from = stoi(argv[1]);
		to = stoi(argv[2]);
	}

	vector<int> r = bfs(from, to);

	if (r.empty()) {
		cout << "No solution!" << endl;
	} else {
		cout << (r.size() - 1) << " presses:\n";

		cout << setfill('0') << setw(4) << r[0];
		for (size_t i = 1; i < r.size(); i++)
			cout << " -> " << setfill('0') << setw(4) << r[i];
	}

	return 0;
}
